#ifndef _SERIAL_H_
#define	_SERIAL_H_
#define SERIAL_API  extern "C"__declspec(dllexport)

#define SEIRAL_PORT_SDK_VERSION_V10	0x10
#define CURRENT_SDK_VERSION		SEIRAL_PORT_SDK_VERSION_V10

#define NAME_LEN		32


///////////////////*********Error code definition begin***********////////////////
#define SERIAL_PORT_NO_ERROR				0			//No error
#define	SERIAL_PORT_NOT_INIT				1			//Serial SDK not Initialized
#define	SERIAL_PORT_OPEN_SERIAL_ERROR		2			//Open serial port fail
#define SERIAL_PORT_RECV_OVERTIME			3			//Receive data timeout 
#define SERIAL_PORT_DATA_CHECKOUT_ERROR		4			//Check out data fail
#define SERIAL_PORT_DATA_NOT_ENOUGH			5			//No enough length data
#define SERIAL_PORT_VERSION_MISMATCH		6			//Version match
#define SERIAL_PORT_ALLOC_RESOURCE_ERROR	7			//Alloc resource fail
#define SERIAL_PORT_PROGRAM_EXCEPTION		8			//Program exception
#define SERIAL_PORT_ORDER_ERROR				9			//Function call out of order
#define SERIAL_PORT_MAX_NUM					10			//Open too many serial ports
#define SERIAL_PORT_SET_COMM_PARAM_ERROR	11			//Set serial configuration fail
#define SERIAL_PORT_RECEIVE_THREAD_ERROR	12			//Start receive thread fail
#define SERIAL_PORT_INPUT_PARAMETER_ERROR	17			//Input parameter error
#define SERIAL_PORT_SEARCH_THREAD_FAIL      18          //Start device search thread fail
#define SERIAL_PORT_MESSAGE_THREAD_FAIL     19          //Start message callback thread fail

#define SERIAL_PORT_DATA_ERROR				5001		//Error data
#define SERIAL_PORT_CHANNEL_ERROR			5002		//Error channel
#define SERIAL_PORT_NET_VERSION_ERROR		5003		//Version of structure error
#define SERIAL_PORT_NOT_SUPPORT				5004		//Device does not support this operation
#define SERIAL_PORT_OPERATE_ERROR			5005		//Device operate fail
///////////////////*********Error code definition end***********////////////////

///////////////////*********Command definition***********////////////////

#define 	SERIAL_PORT_GET_DEV_INFO	                4000 //Get device basic information
#define 	SERIAL_PORT_KEYBOARD_UPGRADE	            4001 //Keyboard upgrade

///////////////////*********Structure definition***********////////////////

#define     DS_1005K              950                   //1005K USB keyboard

#define		SERIALNO_LEN				48      //Serial number length

#define MESSAGE_SERIAL_PORT_EXCHANGE_ERR 1 //Serial port heart beat exchange exception

typedef enum tagLOG_LEVEL_ENUM
{
	ENUM_ERROR_LEVEL = 1, 
	ENUM_DEBUG_LEVEL = 2, 
	ENUM_INFO_LEVEL = 3
}LOG_LEVEL_ENUM;

#define MAX_PROTOCOL_NUM	128			//Max protocol number

typedef enum _SERIALPORT_PROTOCOL_TYPE_ENUM
{
	ENUM_SERIALPORT_UNKNOW_PROTOCOL = 0,		//Unknown protocol type
	ENUM_SERIALPORT_NAKED_PROTOCOL = 1,			//Naked data protocol
	ENUM_SERIALPORT_STANDARD_PROTOCOL,			//Private standard protocol
	ENUM_SERIALPORT_ALARMHOST_PROTOCOL,			//Alarm host 1901 serial protocol
	ENUM_SERIALPORT_OTHER_MATRIX_PROTOCOL,		//Third-party matrix protocol
    ENUM_SERIALPORT_KEYBOARD_PROTOCOL,          //Keyboard protocol
	ENUM_SERIALPORT_MAX_PROTOCOL =  MAX_PROTOCOL_NUM,
}SERIALPORT_PROTOCOL_TYPE_ENUM;

typedef struct tagSERIAL_PORT_SERIAL_ADDRESS
{
	WORD	wHostAddress;	//Host addresssender
	BYTE	byRes[2];		
	WORD	wBeginAddress;	//Receiver begin address
	WORD	wEndAddress;	//Receiver end address
	BYTE	byRes2[8];
} SERIAL_PORT_SERIAL_ADDRESS, *LPSERIAL_PORT_SERIAL_ADDRESS;


typedef enum tagSerialDataBit
{
	DataUnknown = -1,			// Unknown
	SeiralData5 =  5,		// 5 bits per byte
	SerialData6 =  6,		// 6 bits per byte
	SerialData7 =  7,		// 7 bits per byte
	SerialData8 =  8		// 8 bits per byte (default)
}SerialDataBit;


typedef struct tagCOM_PARAM
{
	char szDevice[NAME_LEN];	//Serial port name,e.g,"COM1"
	BYTE  byBaudRate;     //Baud rate, 1-110, 2-300, 3-600, 4-1200, 5-2400, 6-4800, 7-9600, 8-14400, 9-19200, 10- 38400, 11-56000, 12-57600, 13- 115200, 14- 128000, 15-256000
	BYTE  byByteSize;    //Byte sizen bits/BYTE  0-51-62-73-8
	BYTE  byParity;      //Parity 0-No parity1-Odd parity2-Even parity
	BYTE  byStopBits;    //Stop bits,0-1 stop bit1-2 stop bits
	SERIALPORT_PROTOCOL_TYPE_ENUM nProtocol;    //Serial protocol type
}COM_PARAM, *LPCOM_PARAM;


typedef struct tagSERIAL_PORT_KEYBOARD_COMMAND_INFO
{
    DWORD dwSize;
    BYTE byCtrlMode; //Command mode1-Window mode2-Camera mode
    BYTE byRes1[3];
    DWORD dwKeyNo; //Key value0-9999,0xffffffff-invalid
    BYTE bySelfDefineKey; //Self define key, 1-F1 press, 2-F1 release, 3-F2 press, 4-F2 release, 5-F3 press, 6-F3 release, 0xff-invalid
    //Joystick value, 1-Left, 2-Right, 3-Up, 4-Down, 5-Stop, 6-Left up
    //7-Left down, 8-Right up, 9-Right Down0xff-invalid
    BYTE byJoystickKey;
    BYTE byJoystickKeySpeedX; //Joystick X axis speed, 1-7, 0xff-invalid
    BYTE byJoystickKeySpeedY; //Joystick Y axis speed, 1-7, 0xff-invalid
    BYTE byJoystickAction; //Joystick action, 1-clockwise, 2-anti-clockwise, 3-stop, 0xff-invalid
    BYTE byJoystickActionSpeed; //Joystick action speed, 1-7, 0xff-invalid
    BYTE byJoyKeyButton; //Joystick button value, 1-press left button, 2-press right button, 0xff-invalid
    BYTE byRes2[29];
}SERIAL_PORT_KEYBOARD_COMMAND_INFO, *LPSERIAL_PORT_KEYBOARD_COMMAND_INFO;

typedef struct tagSERIAL_PORT_DEV_INFO
{
    DWORD dwSize;
    DWORD dwSoftWareVersion; //Software version, e.g. V1.2.3 is 0x01020003
    DWORD dwHardWareVersion; //Hardware version, e.g. V1.2.3 is 0x01020003
    WORD wDevType; //Device type
    BYTE  byRes1[2];
	BYTE  sSerialNumber[SERIALNO_LEN];//Serial number
    BYTE  byRes2[16];
}SERIAL_PORT_DEV_INFO, *LPSERIAL_PORT_DEV_INFO;

///////////////////*********Interface definition***********////////////////
SERIAL_API BOOL __stdcall	SERIAL_PORT_Init();
SERIAL_API BOOL __stdcall	SERIAL_PORT_Cleanup();
SERIAL_API LONG __stdcall	SERIAL_PORT_GetLastError();
SERIAL_API char* __stdcall  SERIAL_PORT_GetErrorMsg(LONG *pErrorNo = NULL);

typedef void (CALLBACK *SERIAL_DATA_CB)(LONG nSerialHandle, BYTE *pDataBuffer, DWORD dwBufLen, void* pUser);
SERIAL_API LONG __stdcall	SERIAL_PORT_OpenComm(LPCOM_PARAM lpStruCommParam, SERIAL_DATA_CB fSerialDataCB, void* pUser);
SERIAL_API BOOL __stdcall	SERIAL_PORT_CloseComm(LONG lSerialHandle);
SERIAL_API BOOL __stdcall	SERIAL_PORT_SetDataCallBack(LONG lSerialHandle, SERIAL_DATA_CB fSerialDataCB, void* pUserData);

SERIAL_API BOOL __stdcall	SERIAL_PORT_SetDeviceConfig(LONG lSerialHandle, DWORD dwCommand, LPSERIAL_PORT_SERIAL_ADDRESS lpStruAddress, void*  lpInBuffer, DWORD dwInBufferSize, void* lpParamBuffer, DWORD dwParamBufferSize);
SERIAL_API BOOL __stdcall	SERIAL_PORT_GetDeviceConfig(LONG lSerialHandle, DWORD dwCommand, LPSERIAL_PORT_SERIAL_ADDRESS lpStruAddress, void*  lpInBuffer, DWORD dwInBufferSize, void*  lpOutBuffer, DWORD dwOutBufferSize, DWORD *lpReturnLen);
SERIAL_API BOOL __stdcall	SERIAL_PORT_ControlDevice(LONG lSerialHandle, DWORD dwCommand, LPSERIAL_PORT_SERIAL_ADDRESS lpStruAddress, void*  lpInBuffer, DWORD dwInBufferSize, void* lpParamBuffer, DWORD dwParamBufferSize);

SERIAL_API BOOL __stdcall	SERIAL_PORT_SetLogToFile(LONG nLogLevel, char const *strLogDir, BOOL bAutoDel);
SERIAL_API LONG __stdcall	SERIAL_PORT_GetSDKVersion();

typedef void (CALLBACK *Hid_DATA_CB)( BYTE *pDevPath, DWORD dwDevPathLen, BYTE byOnOffline, void* pUser);
typedef void (CALLBACK *MESSAGE_DATA_CB)(LONG lSerialHandle, DWORD dwMsgType, BYTE *pDataBuffer, DWORD dwBufLen, void *pUser);

SERIAL_API LONG __stdcall	SERIAL_PORT_StartHidSearch(Hid_DATA_CB fHidDataCB, void* pUser);
SERIAL_API LONG __stdcall	SERIAL_PORT_StopHidSearch();
SERIAL_API BOOL __stdcall	SERIAL_PORT_SetMessageCallBack(MESSAGE_DATA_CB fMessageDataCallBack, void *pUser);
#endif
